package org.greenstone.android.tipple.base;

import android.content.Context;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.location.LocationListener;
import android.os.Bundle;
import java.util.Queue;
import java.util.LinkedList;

/**
 * Singleton which provides the current location from GPS data.
 * 
 * Ideas for improvement: double integrate the LINEAR_ACCELERATION sensor to get more accurate
 * readings, and use the GPS to correct drift.
 */
public class LocationFusion implements LocationListener {

	private static LocationFusion instance;
	private int clients = 0; // the number of activities who currently need sensor data

	// for rolling average:
	private int lat_win = 10, lon_win = 10, alt_win = 100; // altitude varies a lot, so use a large window 
	private double lat_tot = 0, lon_tot = 0, alt_tot = 0;
	private Queue<Double> lat_his = new LinkedList<Double>();
	private Queue<Double> lon_his = new LinkedList<Double>();
	private Queue<Double> alt_his = new LinkedList<Double>();

	private LocationManager mLocationManager = null;
	private Criteria mCriteria;

	public Location lastKnownLocation = null;

	private LocationFusion(LocationManager lm) {
		mLocationManager = lm;
		
		lastKnownLocation = new Location("");
		lastKnownLocation.setLatitude(-37.788382);
		lastKnownLocation.setLongitude(175.316953);
		lastKnownLocation.setAltitude(80);
		
		mCriteria = new Criteria();
		mCriteria.setAccuracy(Criteria.ACCURACY_LOW);
	}

	public static LocationFusion getInstance() {
		if (instance == null) {
			instance = new LocationFusion(
					(LocationManager) Global.activity.getSystemService(Context.LOCATION_SERVICE));
		}
		return instance;
	}

	/**
	 * Tells the LocationFusion object that there is someone who would like to know the location.
	 * When there are no clients the LocationFusion object removes its event listeners.
	 */
	public void addClient(String name) {
		if (clients++ == 0) {
			registerListeners();
		}
	}

	/**
	 * Tells the LocationFusion object that someone no longer needs the location. When there are no
	 * clients the LocationFusion object removes its event listeners.
	 */
	public void removeClient(String name) {
		if (clients < 0) {
			throw new RuntimeException("LocationFusion: removed too many clients");
		}

		if (--clients == 0) {
			unregisterListeners();
		}
	}

	// Registers a sensor listener for the GPS.
	private void registerListeners() {
		System.err.println("LocationFusion::registerListeners()");
		mLocationManager.requestLocationUpdates(mLocationManager.getBestProvider(mCriteria, true),
				0, 0, this);
	}

	private void unregisterListeners() {
		System.err.println("LocaitonFusion::unregisterListeners()");
		mLocationManager.removeUpdates(this);
	}
	
	@Override
	public void onLocationChanged(Location loc) {
		if (loc != null) {
			lastKnownLocation = loc;
			System.out.println("Real GPS: lat=" + loc.getLatitude() + ", lon=" + loc.getLongitude() + ", alt=" + loc.getAltitude());
			calcRollingAverage();
			System.out.println("Avrg GPS: lat=" + loc.getLatitude() + ", lon=" + loc.getLongitude() + ", alt=" + loc.getAltitude());
		}
	}
	
	private void calcRollingAverage() {
		lat_his.add(lastKnownLocation.getLatitude());
		lon_his.add(lastKnownLocation.getLongitude());
		alt_his.add(lastKnownLocation.getAltitude());
		lat_tot += lastKnownLocation.getLatitude();
		lon_tot += lastKnownLocation.getLongitude();
		alt_tot += lastKnownLocation.getAltitude();

		while (lat_his.size() > lat_win) {
			lat_tot -= lat_his.remove();
		}
		while (lon_his.size() > lon_win) {
			lon_tot -= lon_his.remove();
		}
		while (alt_his.size() > alt_win) {
			alt_tot -= alt_his.remove();
		}

		lastKnownLocation.setLatitude(lat_tot / lat_his.size());
		lastKnownLocation.setLongitude(lon_tot / lon_his.size());
		lastKnownLocation.setAltitude(alt_tot / alt_his.size());
	}

	@Override
	public void onProviderDisabled(String provider) {
	}

	@Override
	public void onProviderEnabled(String provider) {
	}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
	}
}
